<?php

include(plugin_dir_path( __FILE__ ).'andaman_contacts.php');


/**
 * Add function to widgets_init that'll load our widget.
 */

add_action('widgets_init', 'andaman_soc_link_widget');

function andaman_soc_link_widget() {
	register_widget('Andaman_Soc_Link_Widget');
}


/**
 * Widget class.
 * This class handles everything that needs to be handled with the widget:
 * the settings, form, display, and update. 
 *
 */
class Andaman_Soc_Link_Widget extends WP_Widget {

	/**
	 * Widget setup.
	 */		
	
	function __construct() {
		parent::__construct(
			'andaman-soc-link-widget',
			__( 'Social Links', 'andaman' ),
			array(
				'classname' => 'andaman-soc-link-widget',
				'description' => esc_html__( 'Add yours social links.', 'andaman' ),
				'customize_selective_refresh' => true
			)
		);
	}

	/*-----------------------------------------------------------------------------------*/
	/*	Display Widget
	/*-----------------------------------------------------------------------------------*/
	
	function widget( $args, $instance ) {
		extract( $args );
		
		// Our variables from the widget settings
		$title = apply_filters('widget_title', $instance['title'] );
		$facebook = $instance['facebook'];
		$twitter = $instance['twitter'];
		$google = $instance['google'];
		$instagram = $instance['instagram'];
		$linkedin = $instance['linkedin'];
		$youtube = $instance['youtube'];
		$dribbble = $instance['dribbble'];
		$behance = $instance['behance'];
		$tumblr = $instance['tumblr'];

		$time_id = rand();

		/* Before widget (defined by themes). */
		// Before widget (defined by theme functions file)
	echo $before_widget;
	// Display the widget title if one was input
	if ( $title )
		echo $before_title . $title . $after_title;
		?>

		<div class="andaman_about_widget">
            <ul class="soc-footer">
                  <?php if($facebook != false) {?><li><a target="_blank" href="<?php echo  $instance['facebook']?>"><i class="fab fa-facebook-f"></i></a></li><?php } ?>
                  <?php if($twitter != false) {?><li><a target="_blank" href="<?php echo  $instance['twitter']?>"><i class="fab fa-twitter"></i></a></li><?php } ?>
                  <?php if($google != false) {?><li><a target="_blank" href="<?php echo  $instance['google']?>"><i class="fab fa-google-plus-g"></i></a></li><?php } ?>
                  <?php if($instagram != false) {?><li><a target="_blank" href="<?php echo  $instance['instagram']?>"><i class="fab fa-instagram"></i></a></li><?php } ?>
                  <?php if($linkedin != false) {?><li><a target="_blank" href="<?php echo  $instance['linkedin']?>"><i class="fab fa-linkedin-in"></i></a></li><?php } ?>
                  <?php if($youtube != false) {?><li><a target="_blank" href="<?php echo  $instance['youtube']?>"><i class="fab fa-youtube"></i></a></li><?php } ?>
                  <?php if($dribbble != false) {?><li><a target="_blank" href="<?php echo  $instance['dribbble']?>"><i class="fab fa-dribbble"></i></a></li><?php } ?>
                  <?php if($behance != false) {?><li><a target="_blank" href="<?php echo  $instance['behance']?>"><i class="fab fa-behance"></i></a></li><?php } ?>
                  <?php if($tumblr != false) {?><li><a target="_blank" href="<?php echo  $instance['tumblr']?>"><i class="fab fa-tumblr"></i></a></li><?php } ?>
            </ul>

        </div>

		<?php

		// After widget (defined by theme functions file)
		echo $after_widget;
	}







/*-----------------------------------------------------------------------------------*/
/*	Update Widget
/*-----------------------------------------------------------------------------------*/
	
function update( $new_instance, $old_instance ) {
	$instance = $old_instance;

	// Strip tags to remove HTML (important for text inputs)
	$instance['title'] = strip_tags( $new_instance['title'] );
	// Stripslashes for html inputs
	$instance['facebook'] = $new_instance['facebook'];
	$instance['twitter'] = $new_instance['twitter'];
	$instance['google'] = $new_instance['google'];
	$instance['instagram'] = $new_instance['instagram'];
	$instance['linkedin'] = $new_instance['linkedin'];
	$instance['youtube'] = $new_instance['youtube'];
	$instance['dribbble'] = $new_instance['dribbble'];
	$instance['behance'] = $new_instance['behance'];
	$instance['tumblr'] = $new_instance['tumblr'];

	return $instance;
}

/*-----------------------------------------------------------------------------------*/
/*	Widget Settings (Displays the widget settings controls on the widget panel)
/*-----------------------------------------------------------------------------------*/
	 
function form( $instance ) {

	// Set up some default widget settings
	$defaults = array(	'title'			=> '',
						'facebook'		=> 'http://facebook.com',
						'twitter'		=> 'http://twitter.com',
						'google'		=> 'http://google.com',
						'instagram'		=> 'http://instagram.com',
						'linkedin'		=> 'http://linkedin.com',
						'youtube'		=> 'http://youtube.com',
						'dribbble'		=> 'http://dribbble.com',
						'behance'		=> 'http://behance.com',
						'tumblr'		=> 'http://tumblr.com',
						
					);
	
	$instance = wp_parse_args((array) $instance, $defaults);
	?>


	<!-- Widget Title: Text Input -->
	<p>
		<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Title:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['title'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'facebook' ); ?>"><?php _e('Facebook:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'facebook' ); ?>" name="<?php echo $this->get_field_name( 'facebook' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['facebook'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'twitter' ); ?>"><?php _e('Twitter:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'twitter' ); ?>" name="<?php echo $this->get_field_name( 'twitter' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['twitter'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'google' ); ?>"><?php _e('Google Plus:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'google' ); ?>" name="<?php echo $this->get_field_name( 'google' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['google'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'instagram' ); ?>"><?php _e('Instagram:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'instagram' ); ?>" name="<?php echo $this->get_field_name( 'instagram' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['instagram'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'linkedin' ); ?>"><?php _e('LinkedIn:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'linkedin' ); ?>" name="<?php echo $this->get_field_name( 'linkedin' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['linkedin'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'youtube' ); ?>"><?php _e('YouTube:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'youtube' ); ?>" name="<?php echo $this->get_field_name( 'youtube' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['youtube'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'dribbble' ); ?>"><?php _e('Dribbble:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'dribbble' ); ?>" name="<?php echo $this->get_field_name( 'dribbble' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['dribbble'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'behance' ); ?>"><?php _e('Behabce:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'behance' ); ?>" name="<?php echo $this->get_field_name( 'behance' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['behance'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'tumblr' ); ?>"><?php _e('Tumblr:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'tumblr' ); ?>" name="<?php echo $this->get_field_name( 'tumblr' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['tumblr'] ), ENT_QUOTES)); ?>" />
	</p>

	<?php
	}
}
?>