<?php
/**
 * Add function to widgets_init that'll load our widget.
 */

add_action('widgets_init', 'andaman_get_in_touch_widget');

function andaman_get_in_touch_widget() {
	register_widget('Andaman_Get_In_Touch_Widget');
}


/**
 * Widget class.
 * This class handles everything that needs to be handled with the widget:
 * the settings, form, display, and update. 
 *
 */
class Andaman_Get_In_Touch_Widget extends WP_Widget {

	/**
	 * Widget setup.
	 */		

	function __construct() {
		parent::__construct(
			'andaman-get-in-touch-widget',
			__( 'Get in touch', 'andaman' ),
			array(
				'classname' => 'andaman-get-in-touch-widget',
				'description' => esc_html__( 'Get in touch with clients', 'andaman' ),
				'customize_selective_refresh' => true
			)
		);
	}



	/*-----------------------------------------------------------------------------------*/
	/*	Display Widget
	/*-----------------------------------------------------------------------------------*/
	
	function widget( $args, $instance ) {
		extract( $args );
		
		// Our variables from the widget settings
		$title = apply_filters('widget_title', $instance['title'] );
		$adress = $instance['adress'];
		$phone = $instance['phone'];
		$email = $instance['email'];
		$weekend = $instance['weekend'];

		$time_id = rand();

		/* Before widget (defined by themes). */
		// Before widget (defined by theme functions file)
	echo $before_widget;
	// Display the widget title if one was input
	if ( $title )
		echo $before_title . $title . $after_title;
		?>

				<ul class="contact-footer">
                  	<?php if($adress != false) {?><li><i class="fas fa-map-marker-alt"></i> <span><?php echo  $instance['adress']?></span></li><?php } ?>
                  	<?php if($phone != false) {?><li><i class="fas fa-phone"></i> <span><?php echo  $instance['phone']?></span></li><?php } ?>
                  	<?php if($email != false) {?><li><i class="far fa-envelope"></i> <span><?php echo  $instance['email']?></span></li><?php } ?>
                  	<?php if($weekend != false) {?><li><i class="fas fa-lock"></i> <span><?php echo  $instance['weekend']?></span></li><?php } ?>
                </ul>   

		<?php

		// After widget (defined by theme functions file)
		echo $after_widget;
	}


/*-----------------------------------------------------------------------------------*/
/*	Update Widget
/*-----------------------------------------------------------------------------------*/
	
function update( $new_instance, $old_instance ) {
	$instance = $old_instance;

	// Strip tags to remove HTML (important for text inputs)
	$instance['title'] = strip_tags( $new_instance['title'] );
	// Stripslashes for html inputs
	$instance['adress'] = $new_instance['adress'];
	$instance['phone'] = $new_instance['phone'];
	$instance['email'] = $new_instance['email'];
	$instance['weekend'] = $new_instance['weekend'];

	return $instance;
}


/*-----------------------------------------------------------------------------------*/
/*	Widget Settings (Displays the widget settings controls on the widget panel)
/*-----------------------------------------------------------------------------------*/
	 
function form( $instance ) {

	// Set up some default widget settings
	$defaults = array(	'title'		=> __( 'Our Contacts' , 'andaman' ),
						'adress'	=> __( 'Adress: 455 Martinson, Los Angeles' , 'andaman' ),
						'phone'		=> __( 'Phone: 8 (043) 567 - 89 - 30' , 'andaman' ),
						'email'		=> __( 'E-mail: support@email.com' , 'andaman' ),
						'weekend'	=> __( 'Weekend: from 9 am to 6 pm' , 'andaman' ),
					);
	
	$instance = wp_parse_args((array) $instance, $defaults);
	?>


	<!-- Widget Title: Text Input -->
	<p>
		<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Title:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['title'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'adress' ); ?>"><?php _e('Adress:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'adress' ); ?>" name="<?php echo $this->get_field_name( 'adress' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['adress'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'phone' ); ?>"><?php _e('Phone:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'phone' ); ?>" name="<?php echo $this->get_field_name( 'phone' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['phone'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'email' ); ?>"><?php _e('E-mail:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'email' ); ?>" name="<?php echo $this->get_field_name( 'email' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['email'] ), ENT_QUOTES)); ?>" />
	</p>
    <p>
		<label for="<?php echo $this->get_field_id( 'weekend' ); ?>"><?php _e('Open/Close:', 'andaman') ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'weekend' ); ?>" name="<?php echo $this->get_field_name( 'weekend' ); ?>" value="<?php echo stripslashes(htmlspecialchars(( $instance['weekend'] ), ENT_QUOTES)); ?>" />
	</p>

	<?php
	}
}
?>